//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
/*  FILENAME:  AALines.h  [revised 17 AUG 90]

    AUTHOR:  Kelvin Thompson

    DESCRIPTION:  Symbols and globals for the anti-aliased line
      renderer.

    #INCLUDED IN:
      AAMain.c -- Calling routine for renderer.
      AATables.c -- Initialization routines for lookup tables.
      AALines.c -- Rendering code.
*/
// Header for antialiased line drawing. Slightly modified
// to work a bit more nicely with UVRip.
// Changes marked by // CHANGED

// CHANGED 4/4/99
// Now works on rgba colours

#ifndef _AALINES
#define _AALINES
/* frame buffer to hold the anti-aliased line */
/* CHANGED: 5/5/98. Now defined elsewhere */
//int xpix;
//int ypix;
//extern RGB *fbuff;
#include "rgba.h"
/* macros to access the frame buffer */
#define PIXADDR(xx,yy) (fbuff+(yy)*xpix+(xx))
#define PIXINC(dx,dy)  ((dy)*xpix+(dx))

/* fixed-point data types and macros */
typedef int FX;
typedef unsigned int UFX;
#define FX_FRACBITS 16  /* bits of fraction in FX format */
#define FX_0        0   /* zero in fixed-point format */
#define FLOAT_TO_FX(flt)  ((FX)((flt)*(1<<FX_FRACBITS)+0.5))

/* some important constants */
#define PI      3.1415926535897932384626433832795028841971693993751
#define SQRT_2  1.4142135623730950488016887242096980785696718753769

/* square-root function globals */
extern UFX *sqrtfunc;
extern int sqrtcells;
extern int sqrtshift;
#define SQRTFUNC(fxval)  (sqrtfunc[ (fxval) >> sqrtshift ])

/* AA globals */
extern float line_r;  /* line radius */
extern float pix_r;   /* pixel radius */
extern FX *coverage;
extern int covercells;
extern int covershift;
#define COVERAGE(fxval) (coverage[ (fxval) >> covershift ])

// CHANGED 5/5/98
// Added definitions for the two anti-alias line function
// calls. Added line radius and alias radius parameters
// to the init function. 
// The framebuffer and resolution are now passed in to
// the anti_line function, as is the color.
void Anti_Line (rgba *fbuff, rgba color, int xpix, int ypix,
				int X1, int Y1, int X2, int Y2 );
void	Anti_Init(float line_rad=1.0, float anti_rad=(float)SQRT_2);
#endif

