//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// EdgeList.cpp: implementation of the CEdgeList class.
//
// Implementation of the edgelist class. Mantains a linked list
// of paired, sorted indices to the UV map. The two indices 
// represent the line.

// Sorry no detailed comments here.

#include "stdafx.h"
#include "UVRip.h"
#include "EdgeList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// Contructor. Set list to empty
CEdgeList::CEdgeList()
{
	n_edges=0;
	edges=current=NULL;
}

// destructor, delete any existing list
CEdgeList::~CEdgeList()
{
	deleteList();
}

// list deletion funtion
void
CEdgeList::deleteList(void)
{
	EdgeNode	*temp=edges;
	current=temp;
	if (edges!=NULL)
		while (current!=NULL)
		{
			current=temp->next;
			delete temp;
			temp=current;
		}
	n_edges=0;
	edges=current=NULL;
}

// Insert a unique line from UV index va to vb into the list
void
CEdgeList::insert(int va, int vb)
{
	int v1=va,v2=vb;
	
	// sort so smallest index is first
	if (v2<v1)
	{
		v1=vb;
		v2=va;
	}
	current=edges;

	// if no list make a new one
	if (!edges)
	{
		edges = new EdgeNode;
		edges->v1=v1;edges->v2=v2;edges->next=NULL;
		n_edges=1;current=edges;
	}
	else // else insert into the ordered list.
	{
		EdgeNode	*prev,*temp;
		
		// If it's the first node in list
		if (edges->v1<v1 && edges->v2 <v2)
		{
			temp = new EdgeNode;
			temp->v1=v1;temp->v2=v2;temp->next=edges;
			edges=temp;
			n_edges++;current=temp;
			return;
		}
		prev=edges;
		current=edges->next;
			while (current!=NULL)
		{
			// If already in here then ignore
			if (current->v1 == v1 && current->v2==v2)
				return;
			// If current is larger then insert before it
			if (current->v1 < v1 && current->v2<v2)
			{
				temp = new EdgeNode;
				temp->v1=v1;temp->v2=v2;temp->next=current;
				prev->next=temp;
				n_edges++;current=temp;
				return;
			}
			current=current->next;
			prev=prev->next;
		}
		// Goes at end of list
		temp = new EdgeNode;
		temp->v1=v1;temp->v2=v2;temp->next=NULL;
		prev->next=temp;
		n_edges++;current=temp;
	}
}
