//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// EdgeList.h: interface for the CEdgeList class.
//
// Header for the edgelist class. Mantains a linked list
// of paired, sorted indices to the UV map. The two indices 
// represent the line.

#if !defined(AFX_EDGELIST_H__C8C7023E_75B4_11D1_816E_000001069287__INCLUDED_)
#define AFX_EDGELIST_H__C8C7023E_75B4_11D1_816E_000001069287__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// Edge data structure.
// Stores two indices to the UV array
struct Edge
{
	int v1,v2;
};

// Node for storing an edge in.
struct	EdgeNode
{
	int	v1,v2;
	EdgeNode	*next;
};

// Class defining a list of edges.
// Public member functions exist to
//	Insert an new edge:		insert(int,int);
//	Delete edge list:		deleteList();
//	return # of edges:		int getNedges();
//	get the current edge:	EdgeNode *getEdge();
//	goto start of list:		setStart();
//	goto next edge in list:	goNext();
class CEdgeList  
{
public:
	CEdgeList();
	virtual ~CEdgeList();

	void	insert(int, int);
	int		getNedges(void) { return n_edges; };
	void	deleteList(void);
	void	setStart(void) { current=edges; };
	EdgeNode *getEdge(void) { return current; };
	void	goNext(void) { if (current!=NULL) current=current->next; }
private:
	int			n_edges;
	EdgeNode	*edges;
	EdgeNode	*current;
};

#endif // !defined(AFX_EDGELIST_H__C8C7023E_75B4_11D1_816E_000001069287__INCLUDED_)
