//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// CStatic derived control to display a preview of the UV map image.
// The control blends the image with the standard trueSpace material
// backround to show the transparency.
#include "stdafx.h"
#include "uvrip.h"
#include "RipWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRipWnd

CRipWnd::CRipWnd()
{
	m_image = m_bgimage = NULL;
	m_height=m_width=0;
}

CRipWnd::~CRipWnd()
{
}


BEGIN_MESSAGE_MAP(CRipWnd, CStatic)
	//{{AFX_MSG_MAP(CRipWnd)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRipWnd message handlers

// Create both the background and foreground images and find the control size.
// Set the both images to the resource IDB_PREVBG.
// The foreground image will be replaced later.
void CRipWnd::createImages()
{
	BITMAP	b;

	if (m_bgimage!=NULL)
		delete [] m_bgimage;
	m_bgimage = new CBitmap;
	if (m_bgimage==NULL)
		return;
	m_bgimage->LoadBitmap(IDB_PREVBG);
		
	m_bgimage->GetBitmap(&b);
	m_width = b.bmWidth;
	m_height = b.bmHeight;

	if (m_image!=NULL)
		delete [] m_image;
	m_image = new CBitmap;
	if (m_image==NULL)
		return;
	m_image->LoadBitmap(IDB_PREVBG);

	GetClientRect(&m_clientrect);
}

// Paint the blended image onto the control.
// If no image is defined them the background image is
// painted due to the setup procedure.
void CRipWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	CDC dcMem;
			
	if (m_bgimage==NULL)
		createImages();
		
	CBrush brush;
	brush.CreateStockObject(BLACK_BRUSH);
	
	dc.FrameRect(&m_clientrect,&brush);

	dcMem.CreateCompatibleDC(&dc);

	CBitmap* pOldBitmap = dcMem.SelectObject(m_image);
	dc.BitBlt(1,1,m_width,m_height, &dcMem,0,0,SRCCOPY);
	dcMem.SelectObject(pOldBitmap);
	// Do not call CStatic::OnPaint() for painting messages
}

// Blend fgimage with the background image to create the
// preview image. This image is stored in m_image for later
// painting. If no image is passed in then the background
// image is used.
void CRipWnd::newImage(rgba * fgimage)
{
	CPaintDC	dc(this);
	CDC	imgDC,spareDC;
	rgba	bgcol;

	if (m_bgimage==NULL)
		createImages();

	imgDC.CreateCompatibleDC(&dc);
	spareDC.CreateCompatibleDC(&dc);

	// Select image bitmap
	CBitmap* pOldBitmap = imgDC.SelectObject(m_image);

	// Draw background on image
	CBitmap* pOldBitmap2 = spareDC.SelectObject(m_bgimage);
	imgDC.BitBlt(0,0,m_width,m_height, &spareDC,0,0,SRCCOPY);
	spareDC.SelectObject(pOldBitmap2);

	// Blend fgimage onto image
	if (fgimage!=NULL)
		for (int y=0;y<m_height-2;y++)
			for (int x=0;x<m_width-2;x++)
			{
				bgcol.colorref(imgDC.GetPixel(x+1,y+1));
				fgimage[x+y*(m_width-2)].blend(bgcol,fgimage[x+y*(m_width-2)].a/255.0f);
				imgDC.SetPixelV(x+1,y+1,fgimage[x+y*(m_width-2)].Brgb());
			}

	imgDC.SelectObject(pOldBitmap);
}

// Return the size of the preview window.
void CRipWnd::getSize(int & wid, int & hgt)
{
	if (m_bgimage==NULL)
		createImages();
	wid = m_width;
	hgt = m_height;
}


// Delete data in DestroyWindow to avoid assertion failure
// as destructor tries to delete data when the class 
// instance is no longer valid
BOOL CRipWnd::DestroyWindow() 
{
	// TODO: Add your specialized code here and/or call the base class
	if (m_bgimage!=NULL)
		delete [] m_bgimage;
	if (m_image!=NULL)
		delete [] m_image;
	m_image = m_bgimage = NULL;	
	return CStatic::DestroyWindow();
}
