//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// TSXDIALOG.cpp : implementation file
//
// Implementation file for the main dialog box. Automatically
// created by Microsoft Developer Studio.

// This is where all the work is done to extract and save
// UV maps and controlling the dialog box.

#include "stdafx.h"
#include "UVRip.h"
#include "EdgeList.h"
#include "rgba.h"
#include "tga.h"
#include "TSXDIALOG.h"
#include "CompDlg.h"
#include "AALines.h"

#include "tsx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//extern int	g_ver;

/////////////////////////////////////////////////////////////////////////////
// CTSXDIALOG dialog
// The Dialog constructor.
// Set the TSX's default values here
CTSXDIALOG::CTSXDIALOG(void)
	: CDialog(CTSXDIALOG::IDD)
{
	// Set default values for everything
	//{{AFX_DATA_INIT(CTSXDIALOG)
	m_x = 400;	// X size
	m_y = 400;	// Y Size
	m_objname = _T("No Object");	// No object yet
	m_remap = FALSE;				// Don't remap
	m_antialias = TRUE;				// Antialias by default
	m_linewidth = 1.0f;				// line width 1
	m_alias = (float)SQRT_2;		// alias by sqrt(2)
	m_fgalpha = 255;
	m_bgalpha = 255;
	//}}AFX_DATA_INIT

	// Set defaults for other variables
	uv=NULL;			// We have no UV map yet
	n_uv=n_edges=0;
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::DoDataExchange(CDataExchange* pDX)
// Moves data between the dialog box and the
// member variables. This function is created
// automatically by Developer and is accessed
// by calling UpdateData()
void CTSXDIALOG::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTSXDIALOG)
	DDX_Control(pDX, IDC_UVRIPPREVIEW, m_preview);
	DDX_Control(pDX, IDC_FGCOL, m_fgcol);
	DDX_Control(pDX, IDC_BGCOL, m_bgcol);
	DDX_Text(pDX, IDC_X, m_x);
	DDX_Text(pDX, IDC_Y, m_y);
	DDX_Text(pDX, IDC_OBJNAME, m_objname);
	DDX_Check(pDX, IDC_REMAPCHECK, m_remap);
	DDX_Check(pDX, IDC_ALIASCHECK, m_antialias);
	DDX_Text(pDX, IDC_LINEWIDTH, m_linewidth);
	DDX_Text(pDX, IDC_ALIASAMOUNT, m_alias);
	DDX_Text(pDX, IDC_FGALPHA, m_fgalpha);
	DDX_Text(pDX, IDC_BGALPHA, m_bgalpha);
	//}}AFX_DATA_MAP
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::Create()
// Called on creation of the dialog box
// Checks if the current selection is 
// a polygon, if it is then process it's
// UV map.
BOOL
CTSXDIALOG::Create()
{
	// Call base class's Create function
	CDialog::Create(CTSXDIALOG::IDD);
	if (isSelectedPoly(tsxGetCurrentSelection()))
			OnUpdate();

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////
// Developer message map code. Made
// automatically, leave it alone.
BEGIN_MESSAGE_MAP(CTSXDIALOG, CDialog)
	//{{AFX_MSG_MAP(CTSXDIALOG)
	ON_BN_CLICKED(IDC_UPDATE, OnUpdate)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_FORECOL, OnForecol)
	ON_BN_CLICKED(IDC_BACKCOL, OnBackcol)
	ON_BN_CLICKED(IDC_ALIASCHECK, OnAliascheck)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPINFGALPHA, OnDeltaposSpinfgalpha)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPINBGALPHA, OnDeltaposSpinbgalpha)
	ON_EN_KILLFOCUS(IDC_FGALPHA, OnKillfocusFgalpha)
	ON_EN_KILLFOCUS(IDC_BGALPHA, OnKillfocusBgalpha)
	ON_BN_CLICKED(IDC_BGCOL, OnBackcol)
	ON_BN_CLICKED(IDC_FGCOL, OnForecol)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTSXDIALOG message handlers


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::OnCancel() 
// Called when the close button is clicked
void CTSXDIALOG::OnCancel() 
{
	// TODO: Add extra cleanup here
	// Make the window go away
	DestroyWindow();
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::OnUpdate()
// Called when the Update button is pressed
// Check if the current object is a polyhedra.
// If it is then read in its UV map and store
// the edge list. This edge list is used to 
// draw the UV image
//
// Algorithm
//
// Read dialog box parameters
// If the currently selected object is a polyhera then
// {
//		Delete any existing UV edge list
//		Extract the face array from the poly
//		Extract and store the # of UV map entries from the poly
//		If the object has no UV map then
//		{
//			Display error dialog box
//			Set UV edge list to NULL
//			return
//		}
//		Extract and store the UV map
//
//		For each face f
//		{
//			For each vertex v in f
//			{
//				Store the UV edge (v,v+1) in the UV edge list
//			}
//		}
// }
// else
// {
//		Display message saying we have no polyhedra selected
//		set UV edgle list to NULL
//		return
// }
// Update dialog box with new information
//
void CTSXDIALOG::OnUpdate() 
{
	CtsxFace		**faces;// pointer to face list
	tsxSOBJ			*obj;	// pointer to object
	tsxPOLYHEDRON	*poly;	// pointer to polyhedra
	char	name[21];		// name of the TS object
	int		nfaces;			// # of faces in object
	// TODO: Add your control notification handler code here
	
	uv=NULL;	// new UV map
	obj = tsxGetCurrentSelection();	// Get the current object
	
	UpdateData(TRUE);	// Grab current settings

	// If we have an Sobj that is a polygon then rip
	// off the UV map.
	// Else show an informative dialog box and return
	if (isSelectedPoly(obj))
	{
		// Get the name of the object and make it a
		// CString for the dialog box
		tsxGNodeGetName((struct tsxGNODE *)obj,name,20);
		m_objname = name;
		
		// Delete old edgelist
		edges.deleteList();

		// Get a polygon pointer to our object
		poly = (tsxPOLYHEDRON *) obj;

		// Extract # of faces and a pointer
		// to the face array
		// We need to walk around the perimeter
		// of each face and draw lines between
		// the UV points at each vertex
		nfaces = tsxPolyhGetNbrFaces(poly);
		faces = tsxPolyhGetFaceptrArray(poly);

		// Extract # of UV map entries and 
		// a pointer to the UV map array
		n_uv = tsxPolyhGetNbrUV(poly);
		
		// If there is no UV map on the object then show 
		// message and return
		if (n_uv==1)
		{
			AfxMessageBox(IDS_NOUVMAP);
			n_edges=0;
			return;
		}

		uv = tsxPolyhGetUVArray(poly);
		
		// For each face
		for (int i=0;i<nfaces;i++)
		{
			// For each edge in the face extract
			// the UV map values for the vertices
			// and add them to the edge list
			for (int j=0;j<faces[i]->nbrVxs-1;j++)
				edges.insert((faces[i]->pFVxs[j]).uvix,(faces[i]->pFVxs[j+1]).uvix);
			// The last edge can't be added from the loop
			// as it's from vert n-1 -> 0
			edges.insert((faces[i]->pFVxs[0]).uvix,(faces[i]->pFVxs[j]).uvix);
		}
		// Remember # of edges
		n_edges=edges.getNedges();
	}
	else
	{
		// We got no polygon here
		n_edges=0;
		// Show message box
		AfxMessageBox(IDS_NOPOLYSELECT);
	}
	// Display object name in dialog box and redraw preview
	UpdateData(FALSE);
	newPreviewImage();
	// All done!
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::OnSave()
// What happens when we hit save.
// For each edge in the edge list draw a line on the UV
// map image.
void CTSXDIALOG::OnSave() 
{
	rgba			*image=NULL;// Image storage
	tsxSOBJ			*obj;		// pointer to object
	tsxPOLYHEDRON	*poly;		// pointer to polyhedra
	char		name[20];		// polys name
	
	// TODO: Add your control notification handler code here
	
	// If we have no polygon stored then display
	// a helpful message and abort.
	if (n_edges==0)
	{
		AfxMessageBox(IDS_NOPOLYSELECT);
		return;
	}

	// Grab info from dialog box
	// If the info is given is bad then return
	if (!UpdateData(TRUE))
		return;

	// Check if the currently selected object is
	// still the one we have in memory.
	obj = tsxGetCurrentSelection();	// Get the current object
	if (isSelectedPoly(obj))
	{
		poly = (tsxPOLYHEDRON *) obj;
		tsxGNodeGetName((struct tsxGNODE *)obj,name,20);
		// Compare object name and # of uv entries
		if (tsxPolyhGetNbrUV(poly)!= n_uv ||
			name != m_objname)
		{
			AfxMessageBox(IDS_WRONGOBJECT);
			return;
		}
	}	

	// Assign memory for new image
	image = new rgba[m_x*m_y];
	
	// If we're out of memory, print a message and 
	// return
	if (image==NULL)
	{
		AfxMessageBox(IDS_BADIMGMEM);
		return;
	}

	drawUVMap(image,m_x,m_y,m_linewidth,m_alias);

	// Time to write the TGA file
	// I'm not really doing this the correct way
	// I should have a Serialize() override in
	// the TGA class and use that for file IO.
	// I originally wrote this under UNIX,
	// so I haven't fully ported it to MFS yet.
	// I'll do it later.

	// File dialog box specs
	CFileDialog		File(FALSE,"tga","*.tga",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Targa Files (*.tga)|*.tga||");
	CString			FileName;	// Filename to save as
	TGA				*tga;		// TGA file
	CFile			ofp;		// MFC File pointer
	CCompDlg		comp_dlg;
	char			comp=NONE;	

	// If we can get a valid filename then save the file
	if (File.DoModal()==IDOK)
	{	
		if (comp_dlg.DoModal()==IDOK)
			comp=RLE;
		FileName = File.GetPathName();	// Get the name
		tga = new TGA(m_x,m_y,image,comp);	// Make the TGA file
		if (tga==NULL)
		{								// Out of memory
			AfxMessageBox(IDS_BADIMGMEM);
			delete [] image;
			return;
		}
		// Open the file
		ofp.Open(FileName,CFile::modeCreate | CFile::modeWrite);
		tga->write(ofp);	// Write it to the file pointer!
		delete [] tga;		// Get rid of the TGA class
		ofp.Close();		// Close the file
	}

	if (image)
		delete [] image;	// delete the image

	if (m_antialias)
	{
		delete [] coverage;		// also delete antialiasing global
		delete [] sqrtfunc;		// matrices
		coverage=NULL;
		sqrtfunc=NULL;
	}
	// We're finished!!!
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::OnForecol()
// Function called when Foreground button is hit
// Opens the color picker dialog. When a colour is
// chosen the foreground colour is updated and 
// the colour swatch on the dialog redrawn
void CTSXDIALOG::OnForecol() 
{
//	unsigned long	color;		// COLORREF to store color
	CColorDialog	colorpick;	// color picker dialog box
	// TODO: Add your control notification handler code here

	// Open the colourpicker dialog box
	// If it returns a valid colour then
	// store it. 
	if (colorpick.DoModal()==IDOK)
	{
		rgba c;
		c.colorref(colorpick.GetColor());
		m_fgcol.SetColor(c);
		m_fgcol.SetAlpha(m_fgalpha);
		m_fgcol.Invalidate();
	}	
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::OnBackcol()
// Function called when Background button is hit
// Opens the color picker dialog. When a colour is
// chosen the background colour is updated and 
// the colour swatch on the dialog redrawn
void CTSXDIALOG::OnBackcol() 
{
//	unsigned long	color;		// COLORREF to store color
	CColorDialog	colorpick;	// color picker dialog box

	// TODO: Add your control notification handler code here
	
	// Open the colourpicker dialog box
	// If it returns a valid colour then
	// store it
	if (colorpick.DoModal()==IDOK)
	{
		rgba c;
		c.colorref(colorpick.GetColor());
		m_bgcol.SetColor(c);
		m_bgcol.SetAlpha(m_bgalpha);
		m_bgcol.Invalidate();
	}	
}

//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::OnAliasCheck()
// Called whenever the Antialias line check box is 
// clicked. This function enables/disables the
// line width and alias amounts when antialias
// is enabled/disabled
void CTSXDIALOG::OnAliascheck() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);	// Get current dialog settings

	if (m_antialias)	// If antialiasing then activate
	{					// width and amount
		((CEdit*) GetDlgItem(IDC_LINEWIDTH))->EnableWindow(TRUE);
		((CEdit*) GetDlgItem(IDC_ALIASAMOUNT))->EnableWindow(TRUE);
	}
	else				// otherwise diable them
	{
		((CEdit*) GetDlgItem(IDC_LINEWIDTH))->EnableWindow(FALSE);
		((CEdit*) GetDlgItem(IDC_ALIASAMOUNT))->EnableWindow(FALSE);
	}
}


//////////////////////////////////////////////////////////////////////////
// CTSXDIALOG::drawUVLine()
// Function to draw UV lines on RGB *image.
// The UV values are converted to image coordinates and either
// AntiLine or drawLine is called depending if antialiased is
// selected or not.
// If requested, invalid UV coordinates are first remapped to [0,1]
void
CTSXDIALOG::drawUVLine(rgba *image, int wid, int hgt, float xa, float ya, float xb, float yb)
{
	int x1,y1,x2,y2;

	// If requested, remap values outside [0,1] to a repeated
	// UV map.
	// It seems that TS actually uses values outside [0,1]
	// in it's UV maps and remaps them internally. So I'll
	// remap them here to where I think they go so they
	// can be seen.
	// If we're not remapping invalid values then don't draw the line

	// A simple way to see this in action is to make a
	// default sphere and then use the UV projection tool
	// to assign it a Spherical UV map. Rip off the UV map
	// with and without remapping and check out the results
	// when applied to the sphere
	if (m_remap)
	{
		// Check x values
		if (xa > 1 || xb > 1)
		{
			xa -= 1;
			xb -= 1;
		}
		else if (xa < 0 || xb < 0)
		{
			xa += 1;
			xb += 1;
		}
		// check y values
		if (ya > 1 || yb > 1)
		{
			ya -=1;
			yb -=1;
		}
		else if (ya < 0 || yb < 0)
		{
			ya += 1;
			yb += 1;
		}
	}
	else	// Do not draw invalid lines
		if (xa>1 || xa<0 || xb>1 || xb<0 ||
			ya>1 || ya<0 || yb>1 || yb<0)
				return;

	// Convert UV coordinates to image coordinates
	x1 = int((wid-1)*xa);
	x2 = int((wid-1)*xb);
	y1 = int((hgt-1)*ya);
	y2 = int((hgt-1)*yb);
	
	if (m_antialias)	// Draw antialiased line
		Anti_Line(image,m_fgcol.GetColor(),wid,hgt,x1,y1,x2,y2);
	else				// Or draw normal line
		rgbadrawLine(image,m_fgcol.GetColor(),wid,hgt,x1,y1,x2,y2);
}

// Do we have a polygon selected
int CTSXDIALOG::isSelectedPoly(tsxSOBJ *obj)
{
	return (tsxSobjGetType(obj)==e_tsxPOLYHEDRON);
}

// Set up initial color values and set ranges for alpha spin controls
BOOL CTSXDIALOG::OnInitDialog() 
{
	CDialog::OnInitDialog();
	rgba	black(0,0,0,255);
	rgba	white(255,255,255,255);
	// TODO: Add extra initialization here
	m_bgcol.SetColor(white);
	m_fgcol.SetColor(black);
	m_fgalpha=m_bgalpha=255;

	((CSpinButtonCtrl *)GetDlgItem(IDC_SPINFGALPHA))->SetRange(0,255);
	((CSpinButtonCtrl *)GetDlgItem(IDC_SPINBGALPHA))->SetRange(0,255);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// What to do when the spin control is spun
void CTSXDIALOG::OnDeltaposSpinfgalpha(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	// TODO: Add your control notification handler code here
	m_fgalpha += (int)pNMUpDown->iDelta;
	m_fgalpha = minmax(m_fgalpha,0,255);
	m_fgcol.SetAlpha(m_fgalpha);
	UpdateData(FALSE);
	*pResult = 0;
}

// What to do when the spin control is spun
void CTSXDIALOG::OnDeltaposSpinbgalpha(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	// TODO: Add your control notification handler code here
	m_bgalpha += (int)pNMUpDown->iDelta;
	m_bgalpha = minmax(m_bgalpha,0,255);
	m_bgcol.SetAlpha(m_bgalpha);
	UpdateData(FALSE);
	*pResult = 0;
}

// Draw the UV map's image
void CTSXDIALOG::drawUVMap(rgba * img, int wid, int hgt, float aawid, float aaalias)
{
	rgba		bgcol;
	EdgeNode	*edge;
	bgcol = m_bgcol.GetColor();
	for (int i=0;i<wid*hgt;i++)
		rgbaSetColor(img[i],bgcol);

	// If we're antialiasing the lines then
	// initialise the required arrays
	if (m_antialias)
		Anti_Init(aawid,aaalias);


	// For each edge draw a line between
	// it's vertices UV points
	// 'edge' holds the current edge.
	// edge->v? is an index into the UV
	// map where we get the coordinates
	// from
	edges.setStart();	// first edge
	while((edge=edges.getEdge())!=NULL)	// for each edge
	{									// draw the line
		drawUVLine(img,wid,hgt,uv[edge->v1].u,uv[edge->v1].v,
			uv[edge->v2].u,uv[edge->v2].v);
		edges.goNext();	// goto next edge
	}
}

// Redraw the preview image and update the preview window
void CTSXDIALOG::newPreviewImage()
{
	rgba	*image=NULL;
	int		x,y;
	float	ratio;

	if (n_edges)
	{
		// Update alpha values. 
		OnKillfocusFgalpha();
		OnKillfocusBgalpha();
		// Get size of preview window. Draw image 2 pixels smaller to
		// allow for an edge around the image
		m_preview.getSize(x,y);
		x-=2;y-=2;
		image = new rgba[x*y];
		
		// If we're out of memory, print a message and 
		// return
		if (image==NULL)
		{
			AfxMessageBox(IDS_BADIMGMEM);
			return;
		}

		// Calculate relative size of anti-aliased lines
		// and draw image
		ratio = x/(float)m_x;
		drawUVMap(image,x,y,m_linewidth*ratio,m_alias*ratio);
	}
	// Update the preview window
	m_preview.newImage(image);
	m_preview.Invalidate();
	RedrawWindow();
	if (image!=NULL)
		delete [] image;
}

// Called when we click out of the alpha control.
// Update the stored alpha value
void CTSXDIALOG::OnKillfocusFgalpha() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	m_fgalpha = minmax(m_fgalpha,0,255);
	m_fgcol.SetAlpha(m_fgalpha);
	UpdateData(FALSE);
}

// Called when we click out of the alpha control.
// Update the stored alpha value
void CTSXDIALOG::OnKillfocusBgalpha() 
{
	// TODO: Add your control notification handler code here
	UpdateData(TRUE);
	m_bgalpha = minmax(m_bgalpha,0,255);
	m_bgcol.SetAlpha(m_bgalpha);
	UpdateData(FALSE);	
}

// When the main dialog is notified of a click
// on the preview window we must redraw the
// preview window then process other commands
BOOL CTSXDIALOG::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// TODO: Add your specialized code here and/or call the base class
	if (((wParam<<8)>>8)==IDC_UVRIPPREVIEW)
	{
		UpdateData(TRUE);
		newPreviewImage();
	}
	return CDialog::OnCommand(wParam, lParam);
}
