//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// Implementation for TGA class.
//.Can write 24 bit TGA files in compressed or uncompressed format.

// Sorry, no detailed comments here.

#include "stdafx.h"
#include "Tga.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

TGA::TGA(short xi, short yi, rgba *i, char comp)
{
	image = i;
	x = xi;
	y = yi;
	header=NULL;
	idf=NULL;
	compressed=comp;col_type=TRUECOL;col_depth=32;idfield=0;
	indexed=0,llx_off=0,lly_off=y;
	img_desc=ORIGIN_L | ORIGIN_T;

	for(int j=0;j<5;j++)
		colour_map_spec[j]=0;
}

void
TGA::write(CFile &ofp)
{
	writeByte(ofp,idfield);		// 1 byte, length of id field
	writeByte(ofp,indexed);		// 1 byte, 1 if indexed, 0 otherwise
	writeByte(ofp,compressed|col_type);// 1 byte, image format
	ofp.Write(colour_map_spec,5); // 5 bytes, color map spec, ignored


	// Image specs
	writeShort(ofp,llx_off);	// 2 bytes, x lower left corner (0)
	writeShort(ofp,lly_off);	// 2 bytes, y lower left corner (ysize)
	writeShort(ofp,x);		// 2 bytes, x size
	writeShort(ofp,y);		// 2 bytes, y size
	writeByte(ofp,col_depth);	// 1 byte, colour depth
	writeByte(ofp,img_desc);	// 1 byte, image descriptor

	// Image id field now if length in byte 0 > 0.

	// Colour Map data, we have none.

	// Image itself.
	writeImage(ofp);//,image,x,y);
}



void
TGA::writeByte(CFile &ofp, int num)
{
	ofp.Write((char *) &num,1);
}

void
TGA::writeShort(CFile &ofp, int num)
{
	short n=short(num);
	ofp.Write((char *) &n,2);
}

void
TGA::writeLong(CFile &ofp, int num)
{
	long n=long(num);
	ofp.Write((char *) &n,4);
}

void
TGA::writeImage(CFile &ofp)
{
	if (compressed)
		RLEData(ofp);
	else
		ofp.Write(image,x*y*4);
}

void
TGA::RLEData(CFile &ofp)
{
	for (short i=0;i<y;i++)
		RLELine(ofp,i);
}

void
TGA::RLELine(CFile &ofp, short line)
{
	// Set line and data pointers to start of line
	RLEline = image + line*x;
	RLEcurr_pixel=0;
	
	
	// Setup initial packet state
	RLEInitPacketState();
	// at head of while loop, curr_pixel has been encoded
	while (RLEcurr_pixel < x-1)
	{	// If we're currently in a raw packet
		if (RLEstate == RAW_PKT)
		{	// If this and next pixel are not equal
			if (compareARGB(RLEcurr_pixel,RLEcurr_pixel+1))
			{	// encode next pixel as raw packet
				RLErep_count++;
				RLEcurr_pixel++;
			}	
			else // we now have a new rle packet
			{	// remove last pixel from coding
				RLErep_count--;
				// write out current raw and start new packet
				RLEWritePacket(ofp);
				if (!RLEInitPacketState())
					break; // break if at end of line
			}
		}
		else // we're in an rle packet
		{	// If this and next pixel are the same
			if (!compareARGB(RLEcurr_pixel,RLEcurr_pixel+1))
			{	// we're still in an rle packet
				RLErep_count++;
				RLEcurr_pixel++;
			}
			else // it's the end of the RLE packet
			{
				// write packet
				RLEWritePacket(ofp);
				// move to next pixel and re-initialize
				RLEcurr_pixel++;
				if (!RLEInitPacketState())
					break; // break if at end of line
			}
		}
		// Packet is full. Write it out and make new one
		if (RLErep_count == 127)
		{
			RLEWritePacket(ofp);
			RLEcurr_pixel++;
			if (!RLEInitPacketState())
				break;
		}
	}
	// final pixel has been encoded
	// write it out
	RLEWritePacket(ofp);
}

char TGA::RLEInitPacketState()
{
	// if we're not at the end of the line
	if (RLEcurr_pixel != x-1)
	{
		if (compareARGB(RLEcurr_pixel,RLEcurr_pixel+1))
			RLEstate = RAW_PKT;
		else
			RLEstate = RLE_PKT;

		RLErep_count = 1;
		RLEdata = RLEline + RLEcurr_pixel;
		RLEcurr_pixel++;
		return TRUE;
	}
	else
	{	// at end of line, encode final pixel as raw packet
		RLEstate = RAW_PKT;
		RLErep_count = 0;
		RLEdata = RLEline + RLEcurr_pixel;
		return FALSE;
	}
}


void TGA::RLEWritePacket(CFile & ofp)
{
	
	if (RLEstate == RAW_PKT)
	{
		ofp.Write(&RLErep_count,1);
		ofp.Write(RLEdata,(RLErep_count+1)*4);
	}
	else
	{
		RLErep_count |= RLEstate;
		ofp.Write(&RLErep_count,1);
		ofp.Write(RLEdata,4);
	}
}
