//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// Header for TGA class. 
//.Can write 24 bit TGA files in compressed or uncompressed format.
#ifndef _TGA_H
#define _TGA_H
#include	<afx.h>
#include	"rgba.h"


// TGA specs taken from Truevision TGA (TM) File Format Specification
// Version 2.0

// As Per Targa specs the following are defined:
//	Byte:	8 bits
//	Short:	16 bits
//	Long:	32 bits
//
//	Bytes are stored little_endian.

// IMAGE DESCRIPTOR DEFINES

// COLOUR DEPTHS
// defined for use in field 5.6 image descriptor, #bits for attributes
#define	ATTRIB_16	1
#define	ATTRIB_24	0
#define	ATTRIB_32	8

// IMAGE ORIGIN
#define ORIGIN_L	0
#define	ORIGIN_R	16
#define	ORIGIN_T	32
#define	ORIGIN_B	0

// Image colour type
#define	PSEUDO		1
#define	TRUECOL		2
#define	MONO		3

// Compression
#define	RLE		8
#define NONE	0

const unsigned char	RAW_PKT = 0;
const unsigned char	RLE_PKT = 0x80;

class TGA {
	private:
		rgba	*image;
		short	x,y;
		int	hsize;
		char 	*header,*idf,colour_map_spec[5];
		char	compressed, col_depth, idfield,indexed,col_type;
		char	img_desc;
		short	llx_off, lly_off;

		void	writeByte(CFile &ofp, int num);
		void	writeShort(CFile &ofp, int num);
		void	writeLong(CFile &ofp, int num);
		void	writeImage(CFile &ofp);//, rgba *image, int x, int y);

		// Run Length Encoding variables and functions
		int RLEcurr_pixel;
		unsigned char RLEstate;
		unsigned char RLErep_count;
		rgba *RLEdata;
		rgba *RLEline;
		char RLEInitPacketState();
		void RLEData(CFile &ofp);
		void RLELine(CFile &ofp, short line);
		void RLEWritePacket(CFile &ofp);
		inline char compareARGB(int a, int b) { return memcmp(RLEline+(a),RLEline+(b),4); };
	public:
		TGA(short x=0, short y=0, rgba *i=NULL, char comp=NONE);
		void write(CFile &ofp);
		void setX(short X) { x = X; }; 
		void setY(short Y) { y = Y; }; 


};

#endif
