//////////////////////////////////////////////////////////////////////////
// UVRip 1.05 by Bendy's 3D Systems
// Copyright (c) 1999
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// The complete GNU public license can be found in this directory.
//
// Explicit permission is given to link this file with the 
// tsxapi.lib library supplied with the trueSpace API
// Software Developers Kit
//////////////////////////////////////////////////////////////////////////
// UVRip.cpp : Defines the initialization routines for the DLL.
//
// Main UVRip implementation file. Created automatically by 
// Microsoft Developer Studio. Here is where the 4 exported 
// trueSpace functions are located as well as code for controlling
// the display of dialog boxes.

#include "stdafx.h"
#include "UVRip.h"
#include "EdgeList.h"
#include "rgba.h"
#include "TSXDIALOG.h"
#include "TSXAbout.h"

#include "tsx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
//	Note!
//
//		If this DLL is dynamically linked against the MFC
//		DLLs, any functions exported from this DLL which
//		call into MFC must have the AFX_MANAGE_STATE macro
//		added at the very beginning of the function.
//
//		For example:
//
//		extern "C" BOOL PASCAL EXPORT ExportedFunction()
//		{
//			AFX_MANAGE_STATE(AfxGetStaticModuleState());
//			// normal function body here
//		}
//
//		It is very important that this macro appear in each
//		function, prior to any calls into MFC.  This means that
//		it must appear as the first statement within the 
//		function, even before any object variable declarations
//		as their constructors may generate calls into the MFC
//		DLL.
//
//		Please see MFC Technical Notes 33 and 58 for additional
//		details.
//

/////////////////////////////////////////////////////////////////////////////
// CUVRipApp

BEGIN_MESSAGE_MAP(CUVRipApp, CWinApp)
	//{{AFX_MSG_MAP(CUVRipApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// The one and only CUVRipApp object
// Control the UVRip dialog box
CUVRipApp theApp;

// Set the TSX to inactive
BOOL CUVRipApp::sm_bIsActive = FALSE;

// Create global TSX ID for use with callbacks
// (we don't use it in this TSX)
int g_tsxid=0;
//int g_ver=0; // TS version #


// Exported TS functions:
//
//	int tsxGetData( tsxData* tsx_data, int tsxid );
//		Called when TSX is loaded from file.
//	int tsxOnLeftClick(void);
//		Called when icon is clicked
//	void tsxOnRightClick(void);
//		Called when icon is rightclicked
//	void tsxDeactivate(void);
//		Called when TSX is unloaded
//
// These four functions are used to initialise and run
// the TSX.
// tsxGetData() is called when the TSX is loaded from file.
// It gives trueSpace information about the TSX and checks
// that we are running the correct version of trueSpace.
// tsxOnLeftClick() is called when the user clicks on the TSXs
// icon. It simply displays the dialog box and returns. The
// dialog box then handles all the work of extracting and 
// saving the UV image.
// tsxOnRightClick() is called when the user right clicks on the
// TSXs icon. It simply displays a modal about box.
// tsxDeactivate() is called when the TSX is unloaded from
// memory. It is used to clean up the environment and 
// deactivate the TSX.
//
// These four functions simply setup the interface with trueSpace.
// The actual code that does the work is defined elsewhere.
// For better or for worse I have put this code with the 
// dialog box definition in tsxdialog.cpp
//
// Note that these four functions must be exported. There
// are various methods of exporting functions. In this case
// they are listed in the file UVRip.def under the EXPORTS
// section as follows:
//	EXPORTS
//	    ; Explicit exports can go here
//	tsxGetData
//	tsxOnLeftClick
//	tsxDeactivate
//	tsxOnRightClick
//


/////////////////////////////////////////////////////////////////////////////
// tsxGetData()
// Gives a little info about the plugin
// and checks the trueSpace version number.
//
// This function must be exported.
// Note that any exported function that uses MFC
// must have the AFX_MANAGE_STATE function call.
int tsxGetData( tsxData* tsx_data, int tsxid )
{
   //Put at beginning of exported fns, in a DLL using shared MFC DLL.
    AFX_MANAGE_STATE(AfxGetStaticModuleState( ));

	// Structure to store TSX information. Must be static.
    static tsxData MyTsxData = {	// Info about the TSX
	"UVRip",					// Its name
	"Brendan Hack",				// Who wrote it
	IDB_BITMAP,					// TSX icon, bitmap resource
 	IDS_TSXMESSAGE				// Message to display, string resource
    };

/*	short	major,minor;	// To store TS Version #

	// NOTE: Requires TS 3.1 to operate. Put check in here.
	// Major specifies major version #
	//	format: version# * 100
	//		EG: ts 3.1 : 310
	//			ts 3.2 : 320
	// Minor specifies minor version #
	// Minor version number indicates beta releases
	// and the like and can usually be ignored.
	tsxGetTsVersionNbrs(&major,&minor);
	if (major < 300)
	{
		// If we are not using trueSpace 3.1 or higher then
		// display a message box and fail.
		AfxMessageBox(IDS_WRONGVER);
		return tsxPLUG_FAILED;
	}

	g_ver=major;*/
	// Return the TSX information
    *tsx_data = MyTsxData;

    // Save tsxid for this extension. For use
	// with Callback functions.
    g_tsxid = tsxid;

	// All done
    return tsxPLUG_DONE;
}


/////////////////////////////////////////////////////////////////////////////
// tsxOnLeftClick()
// Function for Left-click on Xtension button.
// Displays the dialog box and sets the TSX to active.
//
// This function must be exported.
// Note that any exported function that uses MFC
// must have the AFX_MANAGE_STATE function call
int tsxOnLeftClick(void)
{
    //Put at beginning of exported fns, in a DLL using shared MFC DLL.
    AFX_MANAGE_STATE(AfxGetStaticModuleState( )); 

    // Show parameters dialog
    theApp.ShowDialog();

    // Activate
    CUVRipApp::SetActive();

	// Actually, this return value is a lie. The plugin
	// stays active. If I don't do this the plugin 
	// closes when a trueSpace tool is used.
	// If we were using TS 3.2 we could return tsxPLUG_STAYON
    return tsxPLUG_DONE;
}


/////////////////////////////////////////////////////////////////////////////
// tsxOnRightClick()
// Function for Right-click on Xtension button.
// Displays the about box
//
// This function must be exported.
// Note that any exported function that uses MFC
// must have the AFX_MANAGE_STATE function call
void tsxOnRightClick(void)
{
	CTSXAbout	about;	// make an about box.

    //Put at beginning of exported fns, in a DLL using shared MFC DLL.
    AFX_MANAGE_STATE(AfxGetStaticModuleState( )); 

	// Show the about box
	about.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// tsxDeactivate()
// Called when the Xtension deactivates
// If the dialog box is visible the hide it.
// Set the TSX as inactive.
//
// This function must be exported.
// Note that any exported function that uses MFC
// must have the AFX_MANAGE_STATE function call
void tsxDeactivate(void)
{
    //Put at beginning of exported fns, in a DLL using shared MFC DLL.
    AFX_MANAGE_STATE(AfxGetStaticModuleState( ));

    // NOTE: On exiting trueSpace, `CWinApp::ExitInstance' is called before
    //   tsxDeactivate (if the eXtension was active on exiting).
    //   We want to prevent tsxDeactivate from accessing theApp in this case.

    // Remove the window, if still around
    if ( CUVRipApp::IsActive() )
    {
		CUVRipApp::UnsetActive();
		theApp.HideDialog();
    }
}


/////////////////////////////////////////////////////////////////////////////
// CUVRipApp() construction
// Nothing to go into the constructor.
CUVRipApp::CUVRipApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


/////////////////////////////////////////////////////////////////////////////
// CUVRipApp::InitInstance() 
// Called when the TSX is initially installed.
// Creates the dialog box
BOOL CUVRipApp::InitInstance() 
{
	// TODO: Add your specialized code here and/or call the base class
	
	// Make up the dialog box
    m_pDlg = new CTSXDIALOG();
	
	return CWinApp::InitInstance();
}


/////////////////////////////////////////////////////////////////////////////
// CUVRipApp::ExitInstance() 
// Called when the TSX is removed.
// Set the TSX to inactive and delete the dialog box
int CUVRipApp::ExitInstance() 
{
	// TODO: Add your specialized code here and/or call the base class

    // NOTE: On exiting trueSpace, `ExitInstance' is called before
    //   tsxDeactivate (if the eXtension was active on exiting).
    //   We want to prevent tsxDeactivate from accessing theApp in this case.

    // Deactivate
    UnsetActive();

    delete m_pDlg; //destroys window if not already

    return CWinApp::ExitInstance();
}


/////////////////////////////////////////////////////////////////////////////
// CUVRipApp::ShowDialog()
// If the dialog is not displayed then
// display it
void CUVRipApp::ShowDialog()
{
    //Open Dialog if not already (i.e. if it doesn't have a window)
    if (m_pDlg->GetSafeHwnd() == 0)
    {
		m_pDlg->Create(); //Displays the Dialog window
    }
}


/////////////////////////////////////////////////////////////////////////////
// CUVRipApp::HideDialog()
// Hides the dialog box if it is visible
void CUVRipApp::HideDialog()
{
	// If the dialog is showing then hide it
    if ( m_pDlg->GetSafeHwnd() )
		m_pDlg->DestroyWindow();
}


/////////////////////////////////////////////////////////////////////////////
// CUVRipApp::GetDialog()
// Returns a pointer to the dialog box
CTSXDIALOG* CUVRipApp::GetDialog()
{
	return m_pDlg;
}
